// jxcd.h -- Linux specific low level audio CDROM drive control routines.
// Copyright (c) 1997 by James F. Carter.  1997-12-29, g++
// This is a devolution of WorkMan by Steven Grimm <koreth@hyperion.com> 
//	dated 1995-06-04 (no copyright notice).  See jxcd (tcl script) for
//	freeware license statement.  See jxcd.cc for usage details.

#ifndef JXCD_H
#define JXCD_H

#include <Tcl++.h>

class CdromCmd : public TclCommand {
    
    struct CMDTAB {
	char* name;		/*Command subcode*/
	char* args;		/*Argument types (C=optional char*) */
	int (CdromCmd::* fcn)();  /*Function to handle the operation*/
    };
    static const CMDTAB cmdtab[];	//The command dispatch table
    
		//The states that the drive can be in.
    enum cd_modes {
	    UNKNOWN, TRACK_DONE, PLAYING, FORWARD, PAUSED, STOPPED, 
	    EJECTED, MOUNTED
    };
    static const char* cd_modes_a[];	//ASCII translation of modes

		//Actions that this module can do
    enum cd_actions {NOP, GO, PAUSE, RESUME, STOP, EJECT, ERROR};
    static const char* cd_actions_a[];
		//The CD device
    static TclDString device;

    int FD;			//Open file descriptor for the CDROM device.
    cd_modes mode;		//CD status at last status() call.
		//Storage for command arguments.  Given [cdrom whatever X Y Z],
		//the X Y Z go in cargs[0], [1], [2], and if requested by the
		//args codes in struct CMDTAB, they are converted to int in
		//iargs or double in dargs of the same subscript.
    char* cargs[2];
    int iargs[2];
    double dargs[2];

  public:
    CdromCmd(TclCopy& tcl);
    ~CdromCmd() { wmcd_close(); }		//Destructor closes file descr.
  private:
    virtual int Cmd(int argc, char** argv);	//Command dispatcher
    int wmcd_open();	//Open the drive. [cdrom open $device], device optional.
    int wmcd_close();	//Close the file descriptor
    int cdlen();	//Number of frames on media
    int trackcount();	//Nbr of tracks on media
    int trackinfo();	//Get info about a track: [cdrom trackinfo $track]
			//Returns: {minute second frame absframe datatrack}
    int itrackinfo(int track);	//Ditto, given track as an integer.
    int status();	//Store the drive status
			//Returns: {minute second frame absframe track status}
	//Action functions
    int aioctl(int code, void* data);		//Generic ioctl issuer
    int pause();
    int resume();
    int stop();
    int eject();
    int play();		//[cdrom play $start $end], in absolute frames
    int nothing();	//Does nothing, to simplify calling pgm's logic
};


#endif /*JXCD_H*/